/*****************************************************************************
   file         : $Id: polyaccess.h,v 1.1 2006/09/07 14:06:25 nils Exp $
   description  : forward declarations for xml system
   ------------------------------------------------------------------------

   copyright    : (C) 2006 by Nils Springob, Aachen, GERMANY
   email        : nils.springob@nicai-systems.de
   project      : nicai-systems library

 *****************************************************************************/

#ifndef _NICAI_POLYACCESS_H_
#define _NICAI_POLYACCESS_H_

namespace nicai {

template <class T, class C>
class poly_iterator : public std::iterator<std::bidirectional_iterator_tag, T>
{
  private:
    typedef C container_type;
    typedef typename container_type::const_iterator base_iterator_type;
    const container_type * container;
    base_iterator_type pos;
    void align() {while (pos!=container->end() && dynamic_cast<T*>(*pos)==0) pos++;}
    void back_align() {while (dynamic_cast<T*>(*pos)==0) pos--;}
  public:
    poly_iterator(const poly_iterator & rhs)
    : container(rhs.container), pos(rhs.pos) {align();}
    poly_iterator(const container_type & _container, const base_iterator_type & _pos)
    : container(&_container), pos(_pos) {align();}
    poly_iterator & operator++ () {++pos; align(); return *this;}
    poly_iterator operator++ (int) {poly_iterator ret(*this); ++pos; align(); return ret;}
    poly_iterator & operator-- () {--pos; back_align(); return *this;}
    poly_iterator operator-- (int) {poly_iterator ret(*this); --pos; back_align(); return ret;}
    T & operator* () {return static_cast<T &>(**pos);}
    T * operator-> () {return static_cast<T *>(*pos);}
    bool operator!= (const poly_iterator & rhs) const {return pos!=rhs.pos;}
    bool operator== (const poly_iterator & rhs) const {return pos==rhs.pos;}
    base_iterator_type base() {return pos;}
};

template <class T, class C>
class poly_access
{
  private:
    typedef C container_type;
    const container_type & container;
  public:
    typedef T value_type;
    typedef T& reference;
    typedef const T& const_reference;
    typedef T* pointer;
    typedef const T* const_pointer;
    typedef int difference_type;
    typedef unsigned int size_type;
    typedef poly_iterator<T, C> iterator;
    typedef poly_iterator<const T, C> const_iterator;
    typedef std::reverse_iterator<iterator> reverse_iterator;
    typedef std::reverse_iterator<const_iterator> const_reverse_iterator;

    poly_access(const container_type & _container): container(_container) {}
    const_iterator begin() const {return const_iterator(container, container.begin());}
    const_iterator end() const {return const_iterator(container, container.end());}
    iterator begin() {return iterator(container, container.begin());}
    iterator end() {return iterator(container, container.end());}
    reverse_iterator rbegin() {return end();}
    const_reverse_iterator rbegin() const {return end();}
    reverse_iterator rend() {return begin();}
    const_reverse_iterator rend() const {return begin();}

    bool empty() const {return begin()==end();}
    size_type size() const
    {
      size_type tmp(0);
      iterator it(begin());
      while(it!=end())
        ++it,++tmp;
      return tmp;
    }
};

} // namespace

#endif
