/*****************************************************************************
   file         : $Id: element.cpp,v 1.1 2006/09/07 14:06:25 nils Exp $
   description  :
   ------------------------------------------------------------------------

   copyright    : (C) 2006 by Nils Springob, Aachen, GERMANY
   email        : nils.springob@nicai-systems.de
   project      : nicai-systems library

 *****************************************************************************/

#include "nicai/xml/element.h"
#include "nicai/xml/stream.h"

#include <iostream>

namespace nicai {
namespace xml {

element::element(const std::string & _name)
: name(_name)
{

}


void element::addChild(node * child)
{
  childs.push_back(child);
}


void element::addChild(const node & child)
{
  childs.push_back(child.clone());
}


oxmlbasestream & element::tostream(oxmlbasestream & xout) const
{
  xout << tag(name);
  for (int i=0; i<getAttributeCount(); ++i)
    xout << getAttribute(i);
  for (int i=0; i<getChildCount(); ++i)
    xout << getChild(i);
  xout << endtag();
  return xout;
}


element * element::trycreate(const std::string & name, ixmlbasestream & xin, bool createContent)
{
  if (xin.expectNode(tag(name)))
    return initChild(new element(name), xin, createContent);
  return 0;
}


element * element::trycreate(ixmlbasestream & xin, bool createContent)
{
  node * n = xin.peekNext();
  if (n && n->isTag())
  {
    xin.getNext();
    std::string name = ((tag *)n)->getName();
    return initChild(new element(name), xin, createContent);
  }
  return 0;
}


element & element::init(ixmlbasestream & xin, bool doCreateContent)
{
  xin.readAttributes(attributes, false);
  if (doCreateContent)
    createContent(xin);
  return *this;
}


bool element::init(const std::string & name, ixmlbasestream & xin, bool createContent)
{
  if (!xin.expectNode(tag(name)))
    return false;
  init(xin, createContent);
  return true;
}


void element::createContent(ixmlbasestream & xin)
{
  while (true)
  {
    node * n = createNextContent(xin);
    if (n==0)
      break;
    addChild(n);
  }
}


node * element::createNextContent(ixmlbasestream & xin)
{
  std::string name;
  if (xin.readTag(name))
    return createChildElement(name, xin);
  if (xin.expectNode(endtag()))
    return 0;
  if (xin.expectNode(enddocument()))
    return 0;
  return xin.getNext();
}


element * element::createChildElement(const std::string & name, ixmlbasestream & xin)
{
  return &((new element(name))->init(xin, true));
}


//******************************************************************************
//******************************************************************************
//******************************************************************************


documentroot & documentroot::init(ixmlbasestream & xin, bool doCreateContent)
{
  if (doCreateContent)
    createContent(xin);
  return *this;
}


documentroot * documentroot::trycreate(ixmlbasestream & xin, bool createContent)
{
  if (xin.expectNode(document()))
    return initChild(new documentroot(), xin, createContent);
  return 0;
}


oxmlbasestream & documentroot::tostream(oxmlbasestream & xout) const
{
  xout << document(encoding, standalone);
  for (int i=0; i<getChildCount(); ++i)
    xout << getChild(i);
  xout << enddocument();
  return xout;
}



} // namespace
} // namespace
