/*****************************************************************************
   file         : $Id: element.h,v 1.1 2006/09/07 14:06:25 nils Exp $
   description  :
   ------------------------------------------------------------------------

   copyright    : (C) 2006 by Nils Springob, Aachen, GERMANY
   email        : nils.springob@nicai-systems.de
   project      : nicai-systems library

 *****************************************************************************/

#ifdef CVS_INCLUDE_ID
CVS_ID_INC(CVS_INCLUDE_ID, "$Header: /home/cvs/repository/xmlpp/nicai/xml/element.h,v 1.1 2006/09/07 14:06:25 nils Exp $")
#undef CVS_INCLUDE_ID
#endif

#ifndef _NICAI_XML_ELEMENT_H_
#define _NICAI_XML_ELEMENT_H_

#include "nicai/xml/nodes.h"
#include "nicai/polyaccess.h"
#include <sstream>

namespace nicai {
namespace xml {

template <class T>
class poly_childs : public nicai::poly_access<T, elementchilds>
{
  public:
    poly_childs(const elementchilds & container): nicai::poly_access<T, elementchilds>(container) {}
};


template <class T> std::string to_xml(const T & val);
template <class T> T from_xml(const std::string & val);


template<>
inline static
std::string to_xml<std::string>(const std::string & val)
{
  return val;
}


template<>
inline static
std::string from_xml<std::string>(const std::string & val)
{
  return val;
}


template<class T>
inline static
std::string to_xml(const T & val)
{
  std::ostringstream o;
  o << val;
  return o.str();
}


template<class T>
inline static
T from_xml(const std::string & xval)
{
  std::istringstream i(xval.c_str());
  T val;
  i >> val;
  return val;
}


template <class T>
class property
{
    std::string & value;
  public:
    explicit property(std::string & _value) : value(_value) {value=to_xml<T>(T());}
    const T & operator= (const T & val) {value=to_xml<T>(val); return val;}
    operator T() const {return from_xml<T>(value);}
    const T get() {return from_xml<T>(value);}
    const T & set (const T & val) {value=to_xml<T>(val); return val;}
};


class element : public polynode
{
  private:
    std::string name;
    attributemap attributes;
    void addChild(node * child);

  protected:
    elementchilds childs;
    std::string & getAttributeRef(const std::string & attributeName) {return attributes[attributeName];}

    template <class T>
    static T * initChild(T * child, nicai::xml::ixmlbasestream & xin, bool createContent)
    {child->init(xin, createContent); return child;}


    template <class T>
    static T * create(const std::string & name, nicai::xml::ixmlbasestream & xin, bool createContent = false)
    {return initChild(new T(name), xin, createContent);}

  public:
    const elementchilds & getChilds() {return childs;}

    explicit element(const std::string & name);
    virtual element & init(ixmlbasestream & xin, bool createContent = false);
    bool init(const std::string & expectedtagname, ixmlbasestream & xin, bool createContent = false);
    static element * trycreate(const std::string & name, ixmlbasestream & xin, bool createContent = false);
    static element * trycreate(ixmlbasestream & xin, bool createContent = false);

    void createContent(ixmlbasestream & xin);
    node * createNextContent(ixmlbasestream & xin);

    virtual element * clone() const {return new element(*this);}

    int getAttributeCount() const {return attributes.size();}
    const attribute getAttribute(int index) const {return attribute(attributes[index]);}
    void setAttribute(const attribute & attrib) {attributes[attrib.getName()]=attrib.getVal();}
    const std::string & operator[](const std::string & attributeName) const {return attributes[attributeName];}
    std::string & operator[](const std::string & attributeName) {return attributes[attributeName];}
    bool hasAttribute(const std::string & attributeName) {return attributes.contains(attributeName);}

    const node & getChild(int index) const {return *childs[index];}
    node & getChild(int index) {return *childs[index];}
    int getChildCount() const {return childs.size();}
    void addChild(const node & child);

    const std::string & getName() const {return name;}
    virtual oxmlbasestream & tostream(oxmlbasestream & xout) const;

    virtual nicai::xml::element * createChildElement(const std::string & name, nicai::xml::ixmlbasestream & xin);
};


class documentroot : public element
{
    property<std::string> encoding;
    property<std::string> standalone;
  public:
    documentroot & init(ixmlbasestream & xin, bool doCreateContent);
    documentroot() : element(""), encoding(getAttributeRef("encoding")), standalone(getAttributeRef("standalone")) {}
    static documentroot * trycreate(ixmlbasestream & xin, bool createContent = false);
    virtual oxmlbasestream & tostream(oxmlbasestream & xout) const;
};


} // namespace
} // namespace

#endif


